# Copyright (C) DataInz Technologies Pte. Ltd. - All Rights Reserved
# Unauthorized copying of this file, via any medium is strictly prohibited.
# Proprietary and confidential.

import urllib.request
import urllib.error
from .Database import Database

# The class that is responsible for managing a session for sending and receiving
# requests from MarlinDB server.
class MarlindbSession:

    # Constructor
    def __init__(self, hostname, port, username, password, timeout = 30):

        # Name of the host where MarlinDB is running.
        self.hostname = hostname

        # Port number where MarlinDB is listening.
        self.port = port

        # Username for connecting to MarlinDB
        self.username = username

        # Password for connecting to MarlinDB
        self.password = password

        # The identifier of the session
        self.id = -1

        # The timeout interval for each MarlinDB request.
        self.timeout = timeout

        # The temporary views maintained during the session.
        self.temp_views = []

    # Function to select a database
    def getDatabase(self, dbname):
        return Database(dbname, 0, self)

    # Function to close the session
    def close(self):

        # Cleaning up all the temp views.
        for table in range(0, len(self.temp_views)):
            table.delete()
        self.id = -1

# Function to connect to MarlinDB server.
def initSession(hostname, port, connection, username, password, timeout=30):

    # Creating a new session object.
    session = MarlindbSession(hostname=hostname, port=port, username=username, password=password, timeout=timeout)

    # Checking if MarlinDB server is active.
    try:
        urllib.request.urlopen(url="http://" + hostname + ":" + port + "/test/alive").read()
    except urllib.error.HTTPError as err:
        return session

    session.id = 0
    return session


# Copyright (C) DataInz Technologies Pte. Ltd. - All Rights Reserved
# Unauthorized copying of this file, via any medium is strictly prohibited.
# Proprietary and confidential.

# Class representing a table object in MarlinDB
class Table:

    # Constructor
    def __init__(self, tableName, id, session, is_view=False):

        # Name of the table.
        self.tableName = tableName

        # The unique identifier of the Table.
        self.id = id

        # Boolean denoting whether its a table or a view.
        self.is_view = is_view

        # The session to which the table or view belongs.
        self.session = session

    # Function to get records from the table
    def getRecords(self, columns, offset, limit, encoding='json'):
        return True

    # Function to execute aggregate operation on the table.
    # The function returns a new table with the aggregate results.
    def aggregate(self, columns, options, encoding='json'):
        new_table = Table(tableName='', id=-1, session=self.session, is_view=True)
        self.session.temp_views.append(new_table)
        return new_table

    # Function to execute filter operation on the table.
    # The function returns a new table with filtered results
    def filter(self, columns, conditions, encoding='json'):
        new_table = Table(tableName='', id=-1, session=self.session, is_view=True)
        self.session.temp_views.append(new_table)
        return new_table

    # Function to generate a sub view from the given table using the specified offset and limit.
    # The function returns a new table with the filtered results
    def subView(self, columns, offset, limit, encoding='json'):
        new_table = Table(tableName='', id=-1, session=self.session, is_view=True)
        self.session.temp_views.append(new_table)
        return new_table

    # Function to delete a table and its associated components from MarlinDB
    def delete(self):
        return True


from setuptools import setup

setup(name='marlindb',
      version='0.1.0.3',
      description='Python interface for MarlinDB',
      url='http://www.datainz.com',
      author='DataInz Technologies Pvt. Ltd.',
      author_email='johnspaul92@gmail.com',
      license='Proprietary License',
      packages=['marlindb'],
      zip_safe=False)



# Copyright (C) DataInz Technologies Pte. Ltd. - All Rights Reserved
# Unauthorized copying of this file, via any medium is strictly prohibited.
# Proprietary and confidential.

from .Connection import initSession, MarlindbSession
from .Database import Database
from .Table import Table


# Copyright (C) DataInz Technologies Pte. Ltd. - All Rights Reserved
# Unauthorized copying of this file, via any medium is strictly prohibited.
# Proprietary and confidential.

import json
import urllib.request
import urllib.error
from .Table import Table

# Class representing a database object in MarlinDB
class Database:

    # Constructor
    def __init__(self, dbname, id, session):

        # Name of the database.
        self.dbname = dbname

        # Unique id of the database.
        self.id = id

        # The session to which the database object belongs.
        self.session = session


    # Function to query the database. This function is only for testing and will
    # be removed later.
    def getRecords(self, query, encoding = 'json'):
        query_data = {}
        query_data['session_id'] = self.id
        query_data['query'] = query
        query_json = json.dumps(query_data)
        post_data = query_json.encode(encoding='utf-8')
        headers = {}
        headers['Content-Type'] = 'application/json'

        try:
            request = urllib.request.Request(url="http://" + self.session.hostname + ":" + self.session.port + "/test/query",
                                             data=post_data,
                                             headers=headers)
            urllib.request.urlopen(request, timeout=self.session.timeout).read()
        except urllib.error.HTTPError as err:
            return False
        except urllib.error.URLError as err:
            return False
        return True

    # Function to join 2 tables in the database.
    def join(self, tables, columns):

        # Join currently only supports join of a maximum of 2 tables.
        if len(tables) > 2 or len(columns) > 2:
            return None

        #returning the joined table.
        new_table = Table(tableName=tables[0].tableName, id=0, session=self.session, is_view=True)
        self.session.temp_views.append(new_table)
        return new_table

    # Function to get a table from the database.
    def getTable(self, tableName):
        return Table(tableName=tableName, id=0, session=self.session, is_view=False)